@@ -25,7 +25,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::GPS;
 
-$VERSION = '1.14';
+$VERSION = '1.16';
 
 sub ProcessSEI($$);
 
@@ -521,6 +521,8 @@ my $parsePictureTiming; # flag to enable parsing of picture timing information (
     #       0x0345 - Panasonic HC-V7272
     #       0x0414 - Panasonic AG-AF100
     #       0x0591 - various Panasonic DMC models
+    #       0x0802 - Panasonic DMC-TZ60 with GPS information off
+    #       0x0803 - Panasonic DMC-TZ60 with GPS information on
     #       0x3001 - various Sony DSC, HDR, NEX and SLT models
     #       0x3003 - various Sony DSC models
     #       0x3100 - various Sony DSC, ILCE, NEX and SLT models
@@ -708,7 +710,8 @@ sub DecodeScalingMatrices($)
 
 #------------------------------------------------------------------------------
 # Parse H.264 sequence parameter set RBSP (ref 1)
-# Inputs) 0) ExifTool ref, 1) tag table ref, 2) data ref
+# Inputs: 0) ExifTool ref, 1) tag table ref, 2) data ref
+# Notes: All this just to get the image size!
 sub ParseSeqParamSet($$$)
 {
     my ($et, $tagTablePtr, $dataPt) = @_;
@@ -830,7 +833,9 @@ sub ParseSeqParamSet($$$)
 
 #------------------------------------------------------------------------------
 # Parse H.264 picture timing SEI message (payload type 1) (ref 1)
-# Inputs) 0) ExifTool ref, 1) data ref
+# Inputs: 0) ExifTool ref, 1) data ref
+# Notes: this routine is for test purposes only, and not called unless the
+#        $parsePictureTiming flag is set
 sub ParsePictureTiming($$)
 {
     my ($et, $dataPt) = @_;
@@ -887,6 +892,26 @@ sub ParsePictureTiming($$)
 # Process H.264 Supplementary Enhancement Information (ref 1/PH)
 # Inputs: 0) Exiftool ref, 1) dirInfo ref, 2) tag table ref
 # Returns: 1 if we processed payload type 5
+# Payload types:
+#   0 - buffer period
+#   1 - pic timing
+#   2 - pan scan rect
+#   3 - filler payload
+#   4 - user data registered itu t t35
+#   5 - user data unregistered
+#   6 - recovery point
+#   7 - dec ref pic marking repetition
+#   8 - spare pic
+#   9 - sene info
+#  10 - sub seq info
+#  11 - sub seq layer characteristics
+#  12 - sub seq characteristics
+#  13 - full frame freeze
+#  14 - full frame freeze release
+#  15 - full frame snapshot
+#  16 - progressive refinement segment start
+#  17 - progressive refinement segment end
+#  18 - motion constrained slice group set
 sub ProcessSEI($$)
 {
     my ($et, $dirInfo) = @_;
@@ -913,6 +938,7 @@ sub ProcessSEI($$)
             last unless $t == 255;
         }
         return 0 if $pos + $size > $end;
+        $et->VPrint(1,"    (SEI type $type)\n");
         if ($type == 1) {                   # picture timing information
             if ($parsePictureTiming) {
                 my $buff = substr($$dataPt, $pos, $size);
@@ -1029,9 +1055,7 @@ sub ParseH264Video($$)
         $buff .= substr($$dataPt, $pos, $end - $pos);
         if ($verbose > 1) {
             printf $out "  NAL Unit Type: 0x%x (%d bytes)\n",$nal_unit_type, length $buff;
-            my %parms = ( Out => $out );
-            $parms{MaxLen} = 96 if $verbose < 4;
-            HexDump(\$buff, undef, %parms) if $verbose > 2;
+            $et->VerboseDump(\$buff);
         }
         pos($$dataPt) = $pos = $nextPos;
 
@@ -1084,7 +1108,7 @@ information from H.264 video streams.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
