@@ -12,7 +12,7 @@ require Exporter;
 
 use vars qw($VERSION @ISA @EXPORT_OK);
 
-$VERSION = '1.05';
+$VERSION = '1.09';
 @ISA = qw(Exporter);
 @EXPORT_OK = qw(ReadCSV ReadJSON);
 
@@ -23,7 +23,7 @@ my $charset;
 
 #------------------------------------------------------------------------------
 # Read CSV file
-# Inputs: 0) CSV file name or file ref, 1) database hash ref, 2) missing tag value
+# Inputs: 0) CSV file name, file ref or RAF ref, 1) database hash ref, 2) missing tag value
 # Returns: undef on success, or error string
 # Notes: There are various flavours of CSV, but here we assume that only
 #        double quotes are escaped, and they are escaped by doubling them
@@ -33,11 +33,14 @@ sub ReadCSV($$;$)
     my ($file, $database, $missingValue) = @_;
     my ($buff, @tags, $found, $err, $raf, $openedFile);
 
-    if (ref $file eq 'GLOB') {
+    if (UNIVERSAL::isa($file, 'File::RandomAccess')) {
+        $raf = $file;
+        $file = 'CSV file';
+    } elsif (ref $file eq 'GLOB') {
         $raf = new File::RandomAccess($file);
         $file = 'CSV file';
     } else {
-        open CSVFILE, $file or return "Error opening CSV file '$file'";
+        open CSVFILE, $file or return "Error opening CSV file '${file}'";
         binmode CSVFILE;
         $openedFile = 1;
         $raf = new File::RandomAccess(\*CSVFILE);
@@ -76,7 +79,8 @@ sub ReadCSV($$;$)
         if (@tags) {
             # save values for each tag
             for ($i=0; $i<@vals and $i<@tags; ++$i) {
-                next unless length $vals[$i];   # ignore empty entries
+                # ignore empty entries unless missingValue is empty too
+                next unless length $vals[$i] or defined $missingValue and $missingValue eq '';
                 # delete tag (set value to undef) if value is same as missing tag
                 $fileInfo{$tags[$i]} =
                     (defined $missingValue and $vals[$i] eq $missingValue) ? undef : $vals[$i];
@@ -92,11 +96,13 @@ sub ReadCSV($$;$)
                 # terminate at first blank tag name (eg. extra comma at end of line)
                 last unless length $_;
                 @tags or s/^\xef\xbb\xbf//; # remove UTF-8 BOM if it exists
-                /^[-\w]+(:[-\w+]+)?#?$/ or $err = "Invalid tag name '$_'", last;
+                /^[-\w]+(:[-\w+]+)?#?$/ or $err = "Invalid tag name '${_}'", last;
                 push(@tags, $_);
             }
             last if $err;
             @tags or $err = 'No tags found', last;
+            # fix "SourceFile" case if necessary
+            $tags[0] = 'SourceFile' if lc $tags[0] eq 'sourcefile';
         }
     }
     close CSVFILE if $openedFile;
@@ -117,28 +123,39 @@ sub ToUTF8($)
 
 #------------------------------------------------------------------------------
 # Read JSON object from file
-# Inputs: 0) JSON file handle, 1) optional file buffer reference
+# Inputs: 0) RAF reference or undef, 1) optional scalar reference for data
+#            to read before reading from file (ie. the file read buffer)
 # Returns: JSON object (scalar, hash ref, or array ref), or undef on EOF or
 #          empty object or array (and sets $$buffPt to empty string on EOF)
 # Notes: position in buffer is significant
 sub ReadJSONObject($;$)
 {
-    my ($fp, $buffPt) = @_;
+    my ($raf, $buffPt) = @_;
     # initialize buffer if necessary
     my ($pos, $readMore, $rtnVal, $tok, $key, $didBOM);
     if ($buffPt) {
         $pos = pos $$buffPt;
+        $pos = pos($$buffPt) = 0 unless defined $pos;
     } else {
         my $buff = '';
         $buffPt = \$buff;
         $pos = 0;
     }
 Tok: for (;;) {
+        # (didn't spend the time to understand how $pos could be undef, but
+        #  put a test here to be safe because one user reported this problem)
+        last unless defined $pos;
         if ($pos >= length $$buffPt or $readMore) {
+            last unless defined $raf;
             # read another 64kB and add to unparsed data
             my $offset = length($$buffPt) - $pos;
-            $$buffPt = substr($$buffPt, $pos) if $offset;
-            ($fp and read $fp, $$buffPt, 65536, $offset) or $$buffPt = '', last;
+            if ($offset) {
+                my $buff;
+                $raf->Read($buff, 65536) or $$buffPt = '', last;
+                $$buffPt = substr($$buffPt, $pos) . $buff;
+            } else {
+                $raf->Read($$buffPt, 65536) or $$buffPt = '', last;
+            }
             unless ($didBOM) {
                 $$buffPt =~ s/^\xef\xbb\xbf//;  # remove UTF-8 BOM if it exists
                 $didBOM = 1;
@@ -158,7 +175,7 @@ Tok: for (;;) {
             for (;;) {
                 # read "KEY":"VALUE" pairs
                 unless (defined $key) {
-                    $key = ReadJSONObject($fp, $buffPt);
+                    $key = ReadJSONObject($raf, $buffPt);
                     $pos = pos $$buffPt;
                 }
                 # ($key may be undef for empty JSON object)
@@ -166,7 +183,7 @@ Tok: for (;;) {
                     # scan to delimiting ':'
                     $$buffPt =~ /(\S)/g or $readMore = 1, next Tok;
                     $1 eq ':' or return undef;  # error if not a colon
-                    my $val = ReadJSONObject($fp, $buffPt);
+                    my $val = ReadJSONObject($raf, $buffPt);
                     $pos = pos $$buffPt;
                     return undef unless defined $val;
                     $$rtnVal{$key} = $val;
@@ -180,7 +197,7 @@ Tok: for (;;) {
         } elsif ($tok eq '[') { # array
             $rtnVal = [ ] unless defined $rtnVal;
             for (;;) {
-                my $item = ReadJSONObject($fp, $buffPt);
+                my $item = ReadJSONObject($raf, $buffPt);
                 $pos = pos $$buffPt;
                 # ($item may be undef for empty array)
                 push @$rtnVal, $item if defined $item;
@@ -219,44 +236,55 @@ Tok: for (;;) {
 
 #------------------------------------------------------------------------------
 # Read JSON file
-# Inputs: 0) JSON file name or file ref, 1) database hash ref,
+# Inputs: 0) JSON file name, file ref or RAF ref, 1) database hash ref,
 #         2) flag to delete "-" tags, 3) character set
 # Returns: undef on success, or error string
 sub ReadJSON($$;$$)
 {
     local $_;
     my ($file, $database, $missingValue, $chset) = @_;
-    my ($fp, $openedFile);
+    my ($raf, $openedFile);
 
     # initialize character set for converting "\uHHHH" chars
     $charset = $chset || 'UTF8';
-    if (ref $file eq 'GLOB') {
-        $fp = $file;
+    if (UNIVERSAL::isa($file, 'File::RandomAccess')) {
+        $raf = $file;
+        $file = 'JSON file';
+    } elsif (ref $file eq 'GLOB') {
+        $raf = new File::RandomAccess($file);
         $file = 'JSON file';
     } else {
-        open JSONFILE, $file or return "Error opening JSON file '$file'";
+        open JSONFILE, $file or return "Error opening JSON file '${file}'";
         binmode JSONFILE;
-        $fp = \*JSONFILE;
         $openedFile = 1;
+        $raf = new File::RandomAccess(\*JSONFILE);
     }
-    my $obj = ReadJSONObject($fp);
-    close $fp if $openedFile;
+    my $obj = ReadJSONObject($raf);
+    close JSONFILE if $openedFile;
     unless (ref $obj eq 'ARRAY') {
-        ref $obj eq 'HASH' or return "Format error in JSON file '$file'";
+        ref $obj eq 'HASH' or return "Format error in JSON file '${file}'";
         $obj = [ $obj ];
     }
     my ($info, $found);
     foreach $info (@$obj) {
         next unless ref $info eq 'HASH';
-        # assume a SourceFile of '*' if none specified
-        defined $$info{SourceFile} or $$info{SourceFile} = '*';
+        # fix "SourceFile" case, or assume '*' if SourceFile not specified
+        unless (defined $$info{SourceFile}) {
+            my ($key) = grep /^SourceFile$/i, keys %$info;
+            if ($key) {
+                $$info{SourceFile} = $$info{$key};
+                delete $$info{$key};
+            } else {
+                $$info{SourceFile} = '*';
+            }
+        }
         if (defined $missingValue) {
             $$info{$_} eq $missingValue and $$info{$_} = undef foreach keys %$info;
         }
         $$database{$$info{SourceFile}} = $info;
         $found = 1;
     }
-    return $found ? undef : "No valid JSON objects in '$file'";
+    return $found ? undef : "No valid JSON objects in '${file}'";
 }
 
 
@@ -317,7 +345,7 @@ stored as hash lookups of tag name/value for each SourceFile.
 
 =head1 AUTHOR
 
-Copyright 2003-2017, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2018, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
